@@ -175,7 +175,7 @@ func NewDbFromConfig(c *Config, logger *logging.Logger, connectorCallbacks Retry
 		if port == 0 {
 			port = 5432
 		}
-		query["port"] = []string{strconv.FormatInt(int64(port), 10)}
+		query.Set("port", strconv.FormatInt(int64(port), 10))
 
 		if _, err := c.TlsOptions.MakeConfig(c.Host); err != nil {
 			return nil, err
@@ -183,24 +183,24 @@ func NewDbFromConfig(c *Config, logger *logging.Logger, connectorCallbacks Retry
 
 		if c.TlsOptions.Enable {
 			if c.TlsOptions.Insecure {
-				query["sslmode"] = []string{"require"}
+				query.Set("sslmode", "require")
 			} else {
-				query["sslmode"] = []string{"verify-full"}
+				query.Set("sslmode", "verify-full")
 			}
 
 			if c.TlsOptions.Cert != "" {
-				query["sslcert"] = []string{c.TlsOptions.Cert}
+				query.Set("sslcert", c.TlsOptions.Cert)
 			}
 
 			if c.TlsOptions.Key != "" {
-				query["sslkey"] = []string{c.TlsOptions.Key}
+				query.Set("sslkey", c.TlsOptions.Key)
 			}
 
 			if c.TlsOptions.Ca != "" {
-				query["sslrootcert"] = []string{c.TlsOptions.Ca}
+				query.Set("sslrootcert", c.TlsOptions.Ca)
 			}
 		} else {
-			query["sslmode"] = []string{"disable"}
+			query.Set("sslmode", "disable")
 		}
 
 		uri.RawQuery = query.Encode()
@@ -481,7 +481,7 @@ func (db *DB) BulkExec(
 							return nil
 						},
 						retry.Retryable,
-						backoff.NewExponentialWithJitter(1*time.Millisecond, 1*time.Second),
+						backoff.DefaultBackoff,
 						db.GetDefaultRetrySettings(),
 					)
 				}
@@ -546,7 +546,7 @@ func (db *DB) NamedBulkExec(
 								return nil
 							},
 							retry.Retryable,
-							backoff.NewExponentialWithJitter(1*time.Millisecond, 1*time.Second),
+							backoff.DefaultBackoff,
 							db.GetDefaultRetrySettings(),
 						)
 					}
@@ -607,6 +607,7 @@ func (db *DB) NamedBulkExecTx(
 								if err != nil {
 									return errors.Wrap(err, "can't prepare named statement with context in transaction")
 								}
+								defer func() { _ = stmt.Close() }()
 
 								for _, arg := range b {
 									if _, err := stmt.ExecContext(ctx, arg); err != nil {
@@ -623,7 +624,7 @@ func (db *DB) NamedBulkExecTx(
 								return nil
 							},
 							retry.Retryable,
-							backoff.NewExponentialWithJitter(1*time.Millisecond, 1*time.Second),
+							backoff.DefaultBackoff,
 							db.GetDefaultRetrySettings(),
 						)
 					}
@@ -664,7 +665,7 @@ func (db *DB) YieldAll(ctx context.Context, factoryFunc EntityFactoryFunc, query
 		if err != nil {
 			return CantPerformQuery(err, query)
 		}
-		defer rows.Close()
+		defer func() { _ = rows.Close() }()
 
 		for rows.Next() {
 			e := factoryFunc()
@@ -839,13 +840,54 @@ func (db *DB) GetSemaphoreForTable(table string) *semaphore.Weighted {
 	}
 }
 
+// HasTable checks whether a table is present in the database.
+//
+// The first return value indicates whether a table of the given name exists. The second return value contains any
+// errors that occurred during the check. If the error is not nil, the first argument is always false.
+func (db *DB) HasTable(ctx context.Context, table string) (bool, error) {
+	var tableSchemaFunc string
+	switch db.DriverName() {
+	case MySQL:
+		tableSchemaFunc = "DATABASE()"
+	case PostgreSQL:
+		tableSchemaFunc = "CURRENT_SCHEMA()"
+	default:
+		return false, errors.Errorf("unsupported database driver %q", db.DriverName())
+	}
+
+	var hasTable bool
+	err := retry.WithBackoff(
+		ctx,
+		func(ctx context.Context) error {
+			query := db.Rebind("SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA=" + tableSchemaFunc + " AND TABLE_NAME=?")
+			rows, err := db.QueryContext(ctx, query, table)
+			if err != nil {
+				return CantPerformQuery(err, query)
+			}
+			defer func() { _ = rows.Close() }()
+			hasTable = rows.Next()
+			if err := rows.Close(); err != nil {
+				return err
+			}
+			return rows.Err()
+		},
+		retry.Retryable,
+		backoff.DefaultBackoff,
+		db.GetDefaultRetrySettings())
+	if err != nil {
+		return false, errors.Wrapf(err, "can't verify existence of database table %q", table)
+	}
+	return hasTable, nil
+}
+
 func (db *DB) GetDefaultRetrySettings() retry.Settings {
 	return retry.Settings{
 		Timeout: retry.DefaultTimeout,
-		OnRetryableError: func(_ time.Duration, _ uint64, err, lastErr error) {
-			if lastErr == nil || err.Error() != lastErr.Error() {
-				db.logger.Warnw("Can't execute query. Retrying", zap.Error(err))
-			}
+		OnRetryableError: func(elapsed time.Duration, attempt uint64, err, lastErr error) {
+			db.logger.Warnw("Can't execute query. Retrying",
+				zap.Error(err),
+				zap.Duration("after", elapsed),
+				zap.Uint64("attempt", attempt))
 		},
 		OnSuccess: func(elapsed time.Duration, attempt uint64, lastErr error) {
 			if attempt > 1 {
@@ -867,3 +909,9 @@ func (db *DB) Log(ctx context.Context, query string, counter *com.Counter) perio
 		db.logger.Debugf("Finished executing %q with %d rows in %s", query, counter.Total(), tick.Elapsed)
 	}))
 }
+
+var (
+	// Assert TxOrDB interface compliance of the DB and sqlx.Tx types.
+	_ TxOrDB = (*DB)(nil)
+	_ TxOrDB = (*sqlx.Tx)(nil)
+)
