@@ -171,6 +171,10 @@ func TestFilters(t *testing.T) {
 			Entry:   zapcore.Entry{Level: zap.ErrorLevel, Message: "warp core breach"},
 			Context: []zapcore.Field{zap.Int("b", 42)},
 		},
+		{
+			Entry:   zapcore.Entry{Level: zap.ErrorLevel, Message: "msg", LoggerName: "my.logger"},
+			Context: []zapcore.Field{zap.Int("b", 42)},
+		},
 	}
 
 	logger, sink := New(zap.InfoLevel)
@@ -251,6 +255,11 @@ func TestFilters(t *testing.T) {
 			filtered: sink.FilterLevelExact(zap.WarnLevel),
 			want:     logs[9:10],
 		},
+		{
+			msg:      "filter logger name",
+			filtered: sink.FilterLoggerName("my.logger"),
+			want:     logs[11:12],
+		},
 	}
 
 	for _, tt := range tests {
